//---Тема, в которой обсуждается эта разработка: goo.gl/ijbJho---//


//---v1.3.3_Dmitrysh 10-12-18 пост #1917 обновление от 21-04-2020---//
/*
  Попытка реализовать режим фиксированной мощности.
  Получились следующие режимы.
  режим 1 (в скетче PID) - классический ПИД, все как и было изначально
  режим 2 (в скетче Manual) - ручной режим с заданием фиксированной мощности при включении верхнего нагревателя
  режим 3 (в скетче Auto) - автоматический режим с фиксацией мощности НИ после включения ВИ.
  В ручном режиме необходимо задать мощность которую будет держать НИ при включении ВИ
  С автоматическим режимом могут быть косяки, проверить мне не на чем, смотрел в Proteus с виду работает.
  Добавились 2 новые переменные, в скетче подробно описаны.
*/

/* 
  21-04-20 
  Релиз получил номер v1.3.6.
  Функция ПИД поправлена по мотивам поста #3009, добавлено обнуление integra, integra2, p1, p2, d1, d2,
  Исправлена невозможность завершения профиля при задании длительности любого шага равной 0 сек.,
  Немного пересортированы элементы в секциях скетча,
  Есть выбор между обычными (5 штук) кнопками и аналоговой клавиатурой, аналоговая - на А0,
  Состояние для attachInterrupt задать под свой ZCC,
  Есть выбор в какую программу выводить графики в irsp © от Dmitrysh, 
  Или в Serial Port Plotter (https://github.com/CieNTi/serial_port_plotter)
  В скетче расписано, как это делать.
  В программе irsp © от Dmitrysh синхронизируются графики от ноля времени при старте пайки
  Настройки для Serial Port Plotter:  BAUD=9600, POINTS=600, Y STEP=5, MIN=20, MAX=250. 
  Сначала запускаем график кнопкой с треугольником, затем включаем пайку.
  При старте пайки включается охладитель симисторов пином 4 (ACTIVE LEVEL - HIGH)
  После окончания профиля автоматически НЕ включается охладитель платы пином 5 (ACTIVE LEVEL - HIGH), 
  Надо, чтоб включался - раскомментируйте строчку "digitalWrite(Ext_Fan, HIGH);" (в блоке "завершение пайки")
  Включить охладитель платы из режима IDLE можно коротким нажатием клавиши ВПРАВО
  Выключить охладитель платы можно из режима IDLE клавишей CANCEL
*/

#include <EEPROM.h>
#include <LiquidCrystal_I2C.h>
//LiquidCrystal_I2C lcd(0x27, 20, 4);  //вместо (0x27) указываем адрес своего дисплея //закомментировать если дисплей без I2C
LiquidCrystal_I2C lcd(0x3f, 20, 4);
#define resist_keyboard 1 //Раскоментировать, если клавиатура резистивная

//---секция ПИД---//
float integra, integra2;  //интегральные составляющие ВИ и НИ
float e1, p1, d1, e2, p2, d2; //ошибка регулирования, П-составляющая, Д-составляющая для ВИ и НИ соответственно
#define i_min 0.0         //минимум И составляющей
#define i_max 100.0       //максимум И составляющей
double  Input1; //текущая температура(измеренная max6675) ВИ
byte Output1;   //выход ПИД регулятора ВИ
double  Input2; //текущая температура(измеренная max6675) НИ
byte Output2; //выход ПИД регулятора НИ
#define SENSOR_SAMPLING_TIME 250 //частота обновления текущей температуры (1000=1 раз в секунду)
//---секция ПИД---//

//---секция алгоритма Брезенхема---//
int er1 = 1;  //переменная для расчета переполнения в алгоритме Брезенхема для ВИ
int er2 = 1;  //переменная для расчета переполнения в алгоритме Брезенхема для НИ
int reg1;     //промежуточная переменная для алгоритма Брезенхема для ВИ
int reg2;     //промежуточная переменная для алгоритма Брезенхема для НИ
boolean out1; //выход импульса управления ВИ алгоритма Брезенхема
boolean out2; //выход импульса управления НИ алгоритма Брезенхема
#define SENSOR_SAMPLING_TIME 250 //частота обновления текущей температуры (1000=1 раз в секунду)
//---секция алгоритма Брезенхема---//

//---секция кнопок и пинов подключенияя---//
//Назначаем пины кнопок управления
//аналоговая клавиатура на А0
#ifdef resist_keyboard
//#define A_PINS_BASE 100 // номер с которого начинается нумерация наших "псевдо-кнопок".
#define PIN_RIGHT 100
#define PIN_UP 101
#define PIN_DOWN 102
#define PIN_LEFT 103
#define PIN_SELECT 104

struct A_PIN_DESC { // определяем  структуру которой будем описывать какое значение мы ожидаем для каждого псевдо-пина
  byte pinNo; // номер пина
  int expectedValue;// ожидаемое значение
};
A_PIN_DESC expected_values[] = { // ожидаемые значения для псевдо-кнопок
  { PIN_RIGHT, 128},
  { PIN_UP, 800},
  { PIN_DOWN, 525},
  { PIN_LEFT, 335},
  { PIN_SELECT, 0}
};
#define A_PINS_COUNT sizeof(expected_values)/sizeof(A_PIN_DESC) // вычисляем сколько у нас всего "псевдо-кнопок" заданно.
#define A_POSSIBLE_ABERRATION 50 // допустимое отклонение analogRead от ожидаемого значения, при котором псевдо кнопка считается нажатой
bool digitalReadA(byte pinNo) {

  for (byte i = 0; i < A_PINS_COUNT; i++) { // ищем описание нашего всевдо-пина
    A_PIN_DESC pinDesc = expected_values[i]; // берем очередное описание
    if (pinDesc.pinNo == pinNo) { // нашли описание пина?
      int value = analogRead(A0); // производим чтение аналогово входа
      return (abs(value - pinDesc.expectedValue) < A_POSSIBLE_ABERRATION); // возвращаем HIGH если отклонение от ожидаемого не больше чем на A_POSSIBLE_ABERRATION
    }
  }
  return LOW; // если не нашли описания - считаем что пин у нас LOW
}
//Назначаем пины кнопок управления
int upSwitchPin = PIN_UP;
int downSwitchPin = PIN_DOWN;
int cancelSwitchPin = PIN_LEFT;
int okSwitchPin = PIN_SELECT;
int rightSwitchPin = PIN_RIGHT;
#endif

#ifndef resist_keyboard
byte upSwitchPin = A0; //пин кнопки вверх
byte downSwitchPin = A1; //пин кнопки вниз
byte cancelSwitchPin = A2; //пин кнопки отмена или назад
byte okSwitchPin = A3; //пин кнопки ОК или подтверждения
byte rightSwitchPin = 13; //пин кнопки вправо
#endif

//состояние кнопок по умолчанию
boolean upSwitchState = 0;
boolean downSwitchState = 0;
boolean cancelSwitchState = 0;
boolean okSwitchState = 0;
boolean rightSwitchState = 0;

//переменные для кнопок
long ms_button = 0;
boolean  button_state = false;
boolean  button_long_state = false;

//RelayPin "1"-ВЕРХНИЙ нагреватель
//RelayPin "2"-НИЖНИЙ нагреватель
#define RelayPin1 12  //назначаем пин "ВЕРХНЕГО" нагревателя
#define RelayPin2 11  //назначаем пин "НИЖНЕГО" нагревателя
#define buzzerPin  10 //назначаем пин пищалки
#define Int_Fan  4    //назначаем пин охладителя симисторов и контроллера
#define Ext_Fan  5    //назначаем пин охладителя платы

//назначаем пины преобразователей термопар
#define thermoSO  6 //DO=(SO)
#define thermoCLK  7 //SCK=(CLK)
#define thermoCStop  8 //CS MAX6675 "ВЕРХНЕГО" нагревателя
#define thermoCSbott  9 //CS MAX6675 "НИЖНЕГО" нагревателя
//---секция кнопок и пинов подключения---//

//---секция профиля---//
byte currentStep = 1; //текущий шаг профиля
byte Setpoint1; //установленная температура(уставка) ВИ
int Setpoint2; //установленная температура(уставка) НИ
byte profileSteps; //количество шагов профиля
byte rampRateStep[9]; //скорость роста температуры
byte dwellTimerStep[9]; //установленное время перехода на следующий шаг
byte temperatureStep[9]; //заданные температуры для шагов профиля ВИ
byte pwr_TOP;     //максимальная мощность верхнего нагревателя
byte pwr_BOTTOM;  //максимальная мощность нижнего нагревателя
byte kp1; //пропорциональный коэффициент ПИД для ВИ
byte ki1; //интегральный коэффициент ПИД для ВИ
byte kd1; //дифференциальный коэффициент ПИД для ВИ
byte kp2; //пропорциональный коэффициент ПИД для НИ
byte ki2; //интегральный коэффициент ПИД для НИ
byte kd2; //дифференциальный коэффициент ПИД для НИ
byte currentProfile = 1; //текущий профиль
byte editStep = 0; //номер шага профиля который редактируется в данный момент
//---секция профиля---//

//---Секция флагов---//
boolean TopStart = false; //флаг включения ВИ
boolean flag = 0; //флаг для фиксации стартовой температуры и начала работы нагревателей
byte x = 1; //переменная для перехода на нужный шаг при горячей плате
boolean alarmOn = false; //признак предупреждения
boolean updateScreen = true; //признак обновления экрана
//---Секция флагов---//

//---Секция переменных общего назначения---//
int startTemp; //стартовая температура для профиля, когда НИ вышел на уставку, используется для определения шага профиля с которого начинаем
byte setpointRamp; //заданная температура по определённому шагу профиля ВИ, связана с заданной скоростью роста температуры
byte curCount = 0; //знакоместо звездочек (они бегают по LCD в режиме RUN)
byte counter; //счётчик для рампы и счетчик для времени нахождения в шаге
unsigned long previousMillis; //это для счетчиков
unsigned long nextRead2; //переменная для обновления текущей температуры и расчёта ПИД
unsigned long nextRead1; //переменная для движения "звёздочек" при пайке и подготовки данных для передачи на ПК
int SP2; //переменная для вывода температуры НИ на дисплей
int tc1; //переменная для хранения текущей температуры ВИ
int tc2; //переменная для хранения текущей температуры НИ
//раскомментировать строку ниже, если вывод в "serial port plotter"
//byte Secs = 0; //это для графиков на ПК
int i = 0;
#define GRAPHICS_SAMPLING_TIME 500 //скорость движения "звёздочек" при пайке
byte bh_power;//переменная для хранения мощности НИ, по ее значению определяется режим работы
//0..100 - ручной режим задания можности
//250 - обычный ПИД
// не 0-100 и не 250 - автоматический режим, я поставил 150 на этот режим

char bh_power_auto = 0; //признак режима работы нижнего нагревателя:
//0 - обычный ПИД
//1 - ручной режим с заданием мощности НИ на уставке
//2 - автоматический режим с поддержанием мощности с которой вышли на уставку
//---Секция переменных общего назначения---//

//---структура статусов событий---//
// ***** TYPE DEFINITIONS *****
typedef enum REFLOW_STATE : byte
{
  REFLOW_STATE_IDLE,//экран в режиме простоя, выбор номера профиля
  REFLOW_STATE_MENU_STEPS,//меню выбора числа шагов редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_HEAT,//меню выбора уставки НИ редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_PWR,//меню выбора мощности НИ редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_PWR_AUTO,
  REFLOW_STATE_MENU_BOTTOM_AUTO_PWR,
  REFLOW_STATE_MENU_STEP_RAMP,//меню выбора скорости роста t° ВИ на конкретном шаге редактируемого профиля
  REFLOW_STATE_MENU_STEP_TARGET,//меню выбора конечной t° на конкретном шаге редактируемого профиля
  REFLOW_STATE_MENU_TOP_PWR,//меню выбора мощности ВИ редактируемого профиля
  REFLOW_STATE_MENU_STEP_DWELL,//меню выбора времени удержания конечной t° на конкретном шаге редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_P,//меню выбора коэффициента P для НИ редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_I,//меню выбора коэффициента I для НИ редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_D,//меню выбора коэффициента D для НИ редактируемого профиля
  REFLOW_STATE_MENU_TOP_P,//меню выбора коэффициента P для ВИ редактируемого профиля
  REFLOW_STATE_MENU_TOP_I,//меню выбора коэффициента I для ВИ редактируемого профиля
  REFLOW_STATE_MENU_TOP_D,//меню выбора коэффициента D для ВИ редактируемого профиля
  REFLOW_STATE_STEP_RAMP,//"процесс пайки", рост температуры с заданной для текущего шага ВИ скоростью
  REFLOW_STATE_STEP,
  REFLOW_STATE_STEP_DWELL,
  REFLOW_STATE_COMPLETE,
  REFLOW_STATE_ERROR
}
reflowState_t;

typedef enum REFLOW_STATUS : byte  //структура статуса пайки, запущен или нет
{
  REFLOW_STATUS_OFF,
  REFLOW_STATUS_ON
}
reflowStatus_t;

reflowStatus_t reflowStatus; //переменная статусов событий
reflowState_t reflowState; //переменная статуса пайки
//---структура статусов событий---//



void loadProfile()  //чтение профиля из EEPROM
{
  profileSteps = EEPROM.read((currentProfile - 1) * 38);
  Setpoint2 = 80 + EEPROM.read((currentProfile - 1) * 38 + 1);
  for (int i = 0; i < 9; i++) {
    rampRateStep[i] = EEPROM.read((currentProfile - 1) * 38 + i + 2);
    dwellTimerStep[i] = EEPROM.read((currentProfile - 1) * 38 + i + 11) * 5;
    temperatureStep[i] = EEPROM.read((currentProfile - 1) * 38 + i + 20);
  }
  kp1 = EEPROM.read((currentProfile - 1) * 38 + 29);
  ki1 = EEPROM.read((currentProfile - 1) * 38 + 30);
  kd1 = EEPROM.read((currentProfile - 1) * 38 + 31);
  kp2 = EEPROM.read((currentProfile - 1) * 38 + 32);
  ki2 = EEPROM.read((currentProfile - 1) * 38 + 33);
  kd2 = EEPROM.read((currentProfile - 1) * 38 + 34);
  pwr_TOP = EEPROM.read((currentProfile - 1) * 38 + 35);
  pwr_BOTTOM = EEPROM.read((currentProfile - 1) * 38 + 36);
  bh_power = EEPROM.read((currentProfile - 1) * 38 + 37);

  if (bh_power <= 100) bh_power_auto = 1;
  else if (bh_power == 250) {
    bh_power_auto = 0;
    bh_power = 0;
  }
  // else {
  //   bh_power_auto = 2;
  //   bh_power = 0;
  // }
  return;
}


void setup()
{
  //Записываем единицы в EEPROM для избавления от глюков. Закомментировать после разового запуска
  //for (int i = 0; i < 1024; i++)
  //EEPROM.write(i, 1);

  Serial.begin(9600);

  //устанавливаем пины кнопок на ВХОД и подключаем подтягивающие резисторы
#ifndef resist_keyboard
  pinMode (upSwitchPin, INPUT_PULLUP); //подключен подтягивающий резистор
  pinMode (downSwitchPin, INPUT_PULLUP); //подключен подтягивающий резистор
  pinMode (cancelSwitchPin, INPUT_PULLUP); //подключен подтягивающий резистор
  pinMode (okSwitchPin, INPUT_PULLUP); //подключен подтягивающий резистор
  pinMode (rightSwitchPin, INPUT_PULLUP); //подключен подтягивающий резистор
#endif
#ifdef resist_keyboard
  pinMode(A0, INPUT);
#endif

  //задаём состояние пинов, к которым подключены MAX6675
  pinMode(thermoCStop, OUTPUT);
  pinMode(thermoCSbott, OUTPUT);
  pinMode(thermoCLK, OUTPUT);
  pinMode(thermoSO, INPUT);

  //устанавливаем пин пищалки на ВЫХОД
  pinMode(buzzerPin, OUTPUT);

  //устанавливаем пины охладителей на ВЫХОД
  pinMode (Int_Fan, OUTPUT);
  pinMode (Ext_Fan, OUTPUT);

  //задаём состояние пинов управления выходами
  pinMode(RelayPin1, OUTPUT);
  pinMode(RelayPin2, OUTPUT);

  //сообщение приветствия
  lcd.begin();
  lcd.clear();
  lcd.setCursor(3, 0);
  lcd.print("ARDUINO REWORK");
  lcd.setCursor(2, 1);
  lcd.print("v1.3.6. irsp.exe");
  lcd.setCursor(2, 2);
  lcd.print("Dmitrysh edition");
  lcd.setCursor(2, 3);
  lcd.print("max6675  lcd2004");

  //Мелодия приветствия
  tone(buzzerPin, 523);
  delay(200);
  tone(buzzerPin, 659);
  delay(200);
  tone(buzzerPin, 784);
  delay(200);
  tone(buzzerPin, 1046);
  delay(200);
  noTone(buzzerPin);
  delay(5000);
  lcd.clear();

  loadProfile();//вызов функции loadProfile для загрузки данных профиля из eeprom
  lcd.clear();
  nextRead2 = millis(); //запускаем чтение с термопары ВИ
  //attachInterrupt(0, Dimming, RISING); //настроить порт прерывания(0 или 1) 2й или 3й цифровой пин
  attachInterrupt(0, Dimming, FALLING);
  //attachInterrupt(1, Dimming, RISING); //настроить порт прерывания(0 или 1) 2й или 3й цифровой пин
  /* -------------------------------------------------------------------------------
    LOW вызывает прерывание, когда на порту LOW
    CHANGE прерывание вызывается при смене значения на порту, с LOW на HIGH и наоборот
    RISING прерывание вызывается только при смене значения на порту с LOW на HIGH
    FALLING прерывание вызывается только при смене значения на порту с HIGH на LOW
    ------------------------------------------------------------------------------- */
}


char buf[32]; //это для графиков на ПК


void loop()
{
#ifdef resist_keyboard
  //Считываем состояние кнопок управления
  upSwitchState = digitalReadA(upSwitchPin);
  downSwitchState = digitalReadA(downSwitchPin);
  cancelSwitchState = digitalReadA(cancelSwitchPin);
  okSwitchState = digitalReadA(okSwitchPin);
  rightSwitchState = digitalReadA(rightSwitchPin);
#endif

#ifndef resist_keyboard
  //Считываем состояние кнопок управления для обычных кнопок
  upSwitchState = !digitalRead(upSwitchPin);
  downSwitchState = !digitalRead(downSwitchPin);
  cancelSwitchState = !digitalRead(cancelSwitchPin);
  okSwitchState = !digitalRead(okSwitchPin);
  rightSwitchState = !digitalRead(rightSwitchPin);
#endif

  unsigned long currentMillis = millis();

  int SP2 = Setpoint2;

  //отключил бузер при нажатии на кнопки (нервирует), кому надо раскоментируйте ниже 6 строк
  //  if (upSwitchState == HIGH || downSwitchState == HIGH || cancelSwitchState == HIGH || okSwitchState == HIGH)
  //  {
  //    tone(buzzerPin, 1045);
  //    delay(100);
  //     noTone(buzzerPin);
  //  }


  if (reflowState == REFLOW_STATE_COMPLETE || alarmOn) {
    if (i < 2 && cancelSwitchState == LOW) {
      alarmOn = true;
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      i++;
    }
    else {
      i = 0;
      alarmOn = false;
    }
  }

  switch (reflowState)
  {
    case REFLOW_STATE_IDLE:
      TopStart = false;
      currentStep = 1;
      counter = 0;
      setpointRamp = 0;
      x = 1;             //устанавливаем переменную в исходное состояние
      flag = 0;         //после отановки профиля сбрасываем флаг
      Output1 = 0;
      Output2 = 0;

      if (millis() > nextRead2)
      {
        nextRead2 = millis() + SENSOR_SAMPLING_TIME; //устраняем колебания датчиков температуры
        Input1 = Input1 * 0.8 + 0.2 * (max6675_read_temp (thermoCLK, thermoCStop, thermoSO)); //термопара "ВЕРХНЕГО" нагревателя
        Input2 = Input2 * 0.8 + 0.2 * (max6675_read_temp (thermoCLK, thermoCSbott, thermoSO)); //термопара "НИЖНЕГО"  нагревателя
        tc1 = Input1;
        tc2 = Input2;
        // готовим данные для вывода графиков на ПК,
        //закомментировать две строки ниже, если вывод в "serial port plotter"
        sprintf (buf, "OK%03d%03d%03d%03d%03d\r\n", int(Output1), int(Output2), tc1, tc2, currentProfile);
        Serial.print(buf); //графики на ПК

        //готовим данные для вывода графиков на ПК,
        //раскомментировать строку ниже, если вывод в "serial port plotter"
        //sprintf(buf, "$%d %d;", tc2, tc1);

        lcd.setCursor(16, 2);
        lcd.print("    ");
        lcd.setCursor(16, 2);
        if (isnan(Input1)) {
          lcd.print("Er");
        } else {
          lcd.print(tc1);
        }
        lcd.setCursor(16, 3);
        lcd.print("    ");
        lcd.setCursor(16, 3);
        if (isnan(Input2)) {
          lcd.print("Er");
        } else {
          lcd.print(tc2);
        }
      }
      //Обновляем экран один раз
      if (updateScreen) {

        //Настройка экрана в режиме ожидания
        lcd.clear();
        lcd.setCursor(8, 0);
        lcd.print("IDLE");
        lcd.setCursor(1, 1);
        lcd.print("PTN:");
        lcd.print("      STEP:1 ");
        lcd.setCursor(0, 2);
        lcd.print(" TH  SP:");
        lcd.setCursor(13, 2);
        lcd.print("PV:");
        lcd.setCursor(0, 3);
        lcd.print(" BH  SP:");
        lcd.setCursor(13, 3);
        lcd.print("PV:");
        updateScreen = false;
      }
      lcd.setCursor(5, 1);
      lcd.print(currentProfile);
      lcd.print(" ");
      lcd.setCursor(8, 2);
      lcd.print(temperatureStep[0]);
      lcd.print(" ");
      lcd.setCursor(8, 3);
      lcd.print(SP2);
      lcd.print("  ");

      if (upSwitchState == HIGH && ( millis() - ms_button) > 500) //if up switch is pressed go to next profile
      {
        ms_button =  millis();
        currentProfile = currentProfile + 1;
        if (currentProfile >= 5)//if currentProfile = 4 and up is pressed go back to profile 1
        {
          currentProfile = 1;
        }
        loadProfile();//вызов функции loadProfile для загрузки данных профиля из eeprom
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 500) //same as above just go down one profile
      {
        ms_button =  millis();
        currentProfile = currentProfile - 1;
        if (currentProfile <= 0)
        {
          currentProfile = 4;
        }
        loadProfile();//вызов функции loadProfile для загрузки данных профиля из eeprom
      }


      //фиксируем момент нажатия кнопки "ОК" + защита от дребезга
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
        button_long_state = false;
      }


      //держим "ОК" 3 секунды и заходим в меню настроек
      if (okSwitchState == HIGH && !button_long_state && ( millis() - ms_button) > 3000)
      {
        button_long_state = true;
        button_state = false;
        reflowState = REFLOW_STATE_MENU_STEPS;
        //update next screen
        updateScreen = true;
      }


      //включаем пайку, кнопка сработает после отпускания "ОК" если нажатие меньше 3х секунд
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        tone(buzzerPin, 1045, 500);  //звуковой сигнал при старте профиля
        updateScreen = true;
        //закомментировать две строки ниже, если вывод в "serial port plotter"
        sprintf (buf, "SYNC\r\n");
        Serial.print(buf);
        curCount = 0;
        nextRead1 = millis();
        nextRead2 = millis();
        digitalWrite(Ext_Fan, LOW); //выключить охладитель платы
        integra = 0;
        integra2 = 0;
        p1 = 0;
        p2 = 0;
        d1 = 0;
        d2 = 0;
        reflowStatus = REFLOW_STATUS_ON;
        reflowState = REFLOW_STATE_STEP_RAMP;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        digitalWrite(Ext_Fan, LOW);
      }

      if (rightSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        digitalWrite(Ext_Fan, HIGH);
      }
      break;


    //устанавливаем количество шагов профиля
    case REFLOW_STATE_MENU_STEPS:
      if (updateScreen) {
        lcd.clear();
        lcd.setCursor(3, 0);
        lcd.print("Profile ");
        lcd.print(currentProfile);
        lcd.print(" Edit");
        lcd.setCursor(0, 1);
        lcd.print("                    ");
        lcd.setCursor(2, 2);
        lcd.print("Profile Steps: ");
        lcd.print(profileSteps);
        updateScreen = false;
      }
      lcd.setCursor(17, 2);
      lcd.print(profileSteps);
      lcd.print(" ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profileSteps = profileSteps + 1;
        if (profileSteps >= 10) {
          profileSteps = 1;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profileSteps = profileSteps - 1;
        if (profileSteps <= 0)
        {
          profileSteps = 9;
        }
      }
      if (okSwitchState == LOW && button_long_state) button_long_state = false; //чтобы после входа в меню не кнопка "ОК" не считалась нажатой

      if (okSwitchState == HIGH && !button_state && !button_long_state && ( millis() - ms_button) > 100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        EEPROM.write((currentProfile - 1) * 38, profileSteps);
        reflowState = REFLOW_STATE_MENU_BOTTOM_HEAT;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;


    //устанавливаем температуру "Нижнего Нагревателя"
    case REFLOW_STATE_MENU_BOTTOM_HEAT:

      if (updateScreen) {
        lcd.setCursor(2, 2);
        lcd.print("Bottom Heat:      ");
        updateScreen = false;
      }
      lcd.setCursor(15, 2);
      lcd.print(SP2);
      lcd.print(" ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        Setpoint2 = Setpoint2 + 10;
        if (Setpoint2 >= 330)
        {
          Setpoint2 = 330;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        Setpoint2 = Setpoint2 - 10;
        if (Setpoint2 <= 80)
        {
          Setpoint2 = 80;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        EEPROM.write((currentProfile - 1) * 38 + 1, (Setpoint2 - 80) );
        reflowState = REFLOW_STATE_MENU_BOTTOM_PWR;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;


    //устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
    case REFLOW_STATE_MENU_BOTTOM_PWR:

      if (updateScreen)
      {

        lcd.setCursor(2, 2);
        lcd.print("Bottom PWR:    ");
        updateScreen = false;
      }
      lcd.setCursor(14, 2);
      lcd.print(pwr_BOTTOM);
      lcd.print("%  ");

      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        pwr_BOTTOM = pwr_BOTTOM + 1;
        if (pwr_BOTTOM >= 100)
        {
          pwr_BOTTOM = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();

        if (pwr_BOTTOM >= 1) pwr_BOTTOM = pwr_BOTTOM - 1;
        else  pwr_BOTTOM = 0;
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        EEPROM.write(((currentProfile - 1) * 38 + 36), pwr_BOTTOM);
        reflowState = REFLOW_STATE_MENU_BOTTOM_PWR_AUTO;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;


    //устанавливаем режим управления для НИ (PID, MANUAL, AUTO)
    case REFLOW_STATE_MENU_BOTTOM_PWR_AUTO:

      if (updateScreen)
      {

        lcd.setCursor(1, 2);
        lcd.print("Bottom mode:  ");
        updateScreen = false;
      }
      lcd.setCursor(13, 2);
      switch (bh_power_auto)
      {
        case 0:
          lcd.print("PID   ");
          break;

        case 1:
          lcd.print("Manual");
          break;

        case 2:
          lcd.print("Auto  ");
          break;
      }
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        bh_power_auto = bh_power_auto + 1;
        if (bh_power_auto >= 2)
        {
          bh_power_auto = 2;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        bh_power_auto = bh_power_auto - 1;
        if (bh_power_auto <= 0)
        {
          bh_power_auto = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (bh_power_auto == 0) bh_power = 250;
        if (bh_power_auto == 2) bh_power = 150;
        if (bh_power_auto == 1) reflowState = REFLOW_STATE_MENU_BOTTOM_AUTO_PWR;
        else reflowState = REFLOW_STATE_MENU_TOP_PWR;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    case REFLOW_STATE_MENU_BOTTOM_AUTO_PWR:

      if (updateScreen)
      {
        lcd.setCursor(0, 2);
        lcd.print("  Manual PWR:    ");
        updateScreen = false;
      }
      lcd.setCursor(13, 2);
      lcd.print(bh_power);
      lcd.print("%     ");

      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        bh_power = bh_power + 1;
        if (bh_power >= 100)
        {
          bh_power = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        bh_power = bh_power - 1;
        if (bh_power <= 0)
        {
          bh_power = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        EEPROM.write(((currentProfile - 1) * 9 + 130), bh_power);
        reflowState = REFLOW_STATE_MENU_TOP_PWR;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;


    //устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
    case REFLOW_STATE_MENU_TOP_PWR:

      if (updateScreen)
      {

        lcd.setCursor(2, 2);
        lcd.print("Top     PWR:   ");
        updateScreen = false;
      }
      lcd.setCursor(14, 2);
      lcd.print(pwr_TOP);
      lcd.print("%  ");

      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        pwr_TOP = pwr_TOP + 1;
        if (pwr_TOP >= 100)
        {
          pwr_TOP = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        if (pwr_TOP >= 1) pwr_TOP = pwr_TOP - 1;
        else (pwr_TOP = 0);
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        EEPROM.write(((currentProfile - 1) * 38 + 35), pwr_TOP);
        reflowState = REFLOW_STATE_MENU_STEP_RAMP;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;


    //устанавливаем скорость нагрева "Верхним Нагревателем"
    case REFLOW_STATE_MENU_STEP_RAMP:

      if (updateScreen) {
        lcd.setCursor(2, 2);
        lcd.print("Step ");
        lcd.print(editStep + 1);
        lcd.print("  Ramp:     ");
        updateScreen = false;
      }
      lcd.setCursor(15, 2);
      lcd.print(rampRateStep[editStep]);
      lcd.print(" ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        rampRateStep[editStep] = rampRateStep[editStep] + 1;
        if (rampRateStep[editStep] >= 30)  //(только целые числа) максимальная скорость роста температуры умноженная на 10
        {
          rampRateStep[editStep] = 30;  //(только целые числа) максимальная скорость роста температуры умноженная на 10
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        rampRateStep[editStep] = rampRateStep[editStep] - 1;
        if (rampRateStep[editStep] <= 1)  //(только целые числа) минимальная скорость роста температуры умноженная на 10
        {
          rampRateStep[editStep] = 1;  //(только целые числа) минимальная скорость роста температуры умноженная на 10
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {
          editStep = 0;
          for (byte i = 0; i < 9; i++) EEPROM.write(((currentProfile - 1) * 38 + i + 2), rampRateStep[i] );
          reflowState = REFLOW_STATE_MENU_STEP_TARGET;
        }
        else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        lcd.clear();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;


    //устанавливаем температуру "Верхнего Нагревателя"
    case REFLOW_STATE_MENU_STEP_TARGET:
      if (updateScreen) {
        lcd.setCursor(1, 2);
        lcd.print("Step ");
        lcd.print(editStep + 1);
        lcd.print("  Target: ");
        lcd.print(temperatureStep[editStep]);
        lcd.print("  ");
        updateScreen = false;
      }
      lcd.print("  ");
      lcd.setCursor(16, 2);
      lcd.print(temperatureStep[editStep]);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        temperatureStep[editStep] = temperatureStep[editStep] + 1;
        if (temperatureStep[editStep] >= 255)
        {
          temperatureStep[editStep] = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        temperatureStep[editStep] = temperatureStep[editStep] - 1;
        if (temperatureStep[editStep] <= 0)
        {
          temperatureStep[editStep] = 0;
        }
      }

      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {
          editStep = 0;
          for (int i = 0; i < 9; i++) EEPROM.write(((currentProfile - 1) * 38 + i + 20), temperatureStep[i]);
          reflowState = REFLOW_STATE_MENU_STEP_DWELL;
        }
        else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;


    //устанавливаем время перехода на следующий шаг
    case REFLOW_STATE_MENU_STEP_DWELL:
      if (updateScreen) {
        lcd.setCursor(1, 2);
        lcd.print("Step ");
        lcd.print(editStep + 1);
        lcd.print("   Dwell:");
        lcd.print(dwellTimerStep[editStep]);
        lcd.print("  ");
      }
      lcd.print("  ");
      lcd.setCursor(16, 2);
      lcd.print(dwellTimerStep[editStep]);
      lcd.print("  ");


      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        dwellTimerStep[editStep] = dwellTimerStep[editStep] + 5;
        if (dwellTimerStep[editStep] >= 180)
        {
          dwellTimerStep[editStep] = 180;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();

        if (dwellTimerStep[editStep] >= 5) dwellTimerStep[editStep] = dwellTimerStep[editStep] - 5;
        else dwellTimerStep[editStep] = 0;
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {

          editStep = 0;
          for (int i = 0; i < 9; i++) EEPROM.write(((currentProfile - 1) * 38 + i + 11), dwellTimerStep[i] / 5);
          reflowState = REFLOW_STATE_MENU_BOTTOM_P;
        }
        else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;


    //настройка "ПИД" нижнего нагревателя
    case REFLOW_STATE_MENU_BOTTOM_P:
      if (updateScreen) {
        lcd.setCursor(0, 2);
        lcd.print("                  ");
        lcd.setCursor(3, 2);
        lcd.print("Bottom Heater:  ");
        lcd.setCursor(8, 3);
        lcd.print("P=");
        lcd.print(kp2);
        lcd.print("  ");
        updateScreen = false;
      }
      lcd.setCursor(10, 3);
      lcd.print(kp2);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kp2 = kp2 + 1;
        if (kp2 >= 255)
        {
          kp2 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kp2 = kp2 - 1;
        if (kp2 <= 0)
        {
          kp2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        EEPROM.write(((currentProfile - 1) * 38 + 32), kp2);
        reflowState = REFLOW_STATE_MENU_BOTTOM_I;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_BOTTOM_I:
      lcd.setCursor(8, 3);
      lcd.print("I=");
      lcd.print(ki2);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        ki2 = ki2 + 1;
        if (ki2 >= 255)
        {
          ki2 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        ki2 = ki2 - 1;
        if (ki2 <= 0)
        {
          ki2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 38 + 33), ki2);
        reflowState = REFLOW_STATE_MENU_BOTTOM_D;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_BOTTOM_D:
      lcd.setCursor(8, 3);
      lcd.print("D=");
      lcd.print(kd2);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kd2 = kd2 + 1;
        if (kd2 >= 255)
        {
          kd2 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kd2 = kd2 - 1;
        if (kd2 <= 0)
        {
          kd2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 38 + 34), kd2);
        reflowState = REFLOW_STATE_MENU_TOP_P;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;


    //настройка "ПИД" верхнего нагревателя
    case REFLOW_STATE_MENU_TOP_P:
      if (updateScreen) {
        lcd.setCursor(0, 2);
        lcd.print("                  ");
        lcd.setCursor(5, 2);
        lcd.print("Top Heater:");
        lcd.setCursor(8, 3);
        lcd.print("P=");
        lcd.print(kp1);
        lcd.print("  ");
        updateScreen = false;
      }
      lcd.setCursor(10, 3);
      lcd.print(kp1);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kp1 = kp1 + 1;
        if (kp1 >= 255)
        {
          kp1 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kp1 = kp1 - 1;
        if (kp1 <= 0)
        {
          kp1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        EEPROM.write(((currentProfile - 1) * 38 + 29), kp1);
        reflowState = REFLOW_STATE_MENU_TOP_I;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_TOP_I:
      lcd.setCursor(8, 3);
      lcd.print("I=");
      lcd.print(ki1);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        ki1 = ki1 + 1;
        if (ki1 >= 255)
        {
          ki1 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        ki1 = ki1 - 1;
        if (ki1 <= 0)
        {
          ki1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 38 + 30), ki1);
        reflowState = REFLOW_STATE_MENU_TOP_D;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    case REFLOW_STATE_MENU_TOP_D:
      lcd.setCursor(8, 3);
      lcd.print("D=");
      lcd.print(kd1);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kd1 = kd1 + 1;
        if (kd1 >= 255)
        {
          kd1 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kd1 = kd1 - 1;
        if (kd1 <= 0)
        {
          kd1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 38 + 31), kd1);
        lcd.clear();
        reflowState = REFLOW_STATE_IDLE;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        lcd.clear();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;


    //"процесс пайки", рост температуры с заданной скоростью
    case REFLOW_STATE_STEP_RAMP:
      digitalWrite(Int_Fan, HIGH); //включить охладитель симисторов и контроллера
      if (updateScreen) {
        lcd.setCursor(8, 0);
        lcd.print("RUN!");
        updateScreen = false;
      }

      //if (tc2 >= SP2-2 && !TopStart) TopStart = true;  //если температура низа вышла на уставку, включаем верхний нагреватель
      //if (tc1 >= 118 && !TopStart) TopStart = true; //если температура низа вышла на уставку включаем верхний нагреватель
      if (tc1 >= SP2 - 2 && !TopStart) TopStart = true; //если температура низа вышла на уставку, включаем верхний нагреватель


      if (TopStart == true)  //включаем верхний нагреватель
        //-------------------------------------------------------------------------------------------
      { //открываем скобку (чтоб не запутаться)

        //фиксируем стартовую температуру
        if (flag == 0)
        {
          startTemp = tc1;
          flag = 1;
        }

        //устанавливаем нужный шаг, до которой нагрета плата
        if (startTemp > temperatureStep[currentStep - 1]) {
          for (x = 1; startTemp > temperatureStep[currentStep - 1]; currentStep++) {
            x++;
          }
        }
        if (currentStep > x && flag == 1) {
          flag = 0;
          startTemp = temperatureStep[currentStep - 2];
          flag = 1;
        }
        lcd.setCursor(17, 1);
        lcd.print(currentStep);
        lcd.setCursor(8, 3);
        lcd.print(SP2);


        //счётчик скорости роста температуры
        if ((currentMillis - previousMillis) > 1000 / (rampRateStep[currentStep - 1] * 0.1)) //скорость роста температуры от 0.1с. до 3с.

        {
          previousMillis = currentMillis;
          counter = counter + 1;
          setpointRamp = counter + startTemp;
          lcd.setCursor(8, 2);
          lcd.print("   ");
          lcd.setCursor(8, 2);
          lcd.print(setpointRamp);
          lcd.print("  ");
          Setpoint1 = setpointRamp;
        }
      } //закрывам скобку
      //----------------------------------------------------------------------------------------

      if (setpointRamp >= temperatureStep[currentStep - 1]) //если достигли нужной температуры
      {
        lcd.setCursor(8, 2);
        lcd.print(temperatureStep[currentStep - 1]);
        reflowState = REFLOW_STATE_STEP;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        digitalWrite(Int_Fan, LOW); //выключить охладитель симисторов и контроллера
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
        updateScreen = true;
      }
      break;

    case REFLOW_STATE_STEP:
      Setpoint1 = temperatureStep[currentStep - 1];
      if (Input1 >= temperatureStep[currentStep - 1])
      {
        counter = 0;
        reflowState = REFLOW_STATE_STEP_DWELL;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        digitalWrite(Int_Fan, LOW); //выключить охладитель симисторов и контроллера
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
        updateScreen = true;
      }
      break;


    //считаем время перехода на следующий шаг
    case REFLOW_STATE_STEP_DWELL:
      if (currentMillis - previousMillis > 1000)
      {
        previousMillis = currentMillis;
        counter = counter + 1;
      }
      if (counter >= dwellTimerStep[currentStep - 1]) //если счётчик равен установленному времени
      {
        counter = 0;
        setpointRamp = 0;
        if (profileSteps == currentStep) //если достигли последнего шага
        {
          currentStep = 1;
          x = 1;      //устанавливаем переменную в исходное состояние
          flag = 0;   //после завершения профиля сбрасываем флаг
          reflowState = REFLOW_STATE_COMPLETE;
        }
        else //если шаг не последний
        {
          currentStep++; //переходим на следующий шаг
          reflowState = REFLOW_STATE_STEP_RAMP;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        digitalWrite(Int_Fan, LOW); //выключить охладитель симисторов и контроллера
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
        updateScreen = true;
      }
      break;


    //завершение пайки
    case REFLOW_STATE_COMPLETE:
      //digitalWrite(Ext_Fan, HIGH); //включить охладитель платы
      digitalWrite(Int_Fan, LOW); //выключить охладитель симисторов и контроллера
      reflowStatus = REFLOW_STATUS_OFF;
      reflowState = REFLOW_STATE_IDLE;
      updateScreen = true;
      break;
  }


  //включение нагревателей
  if (reflowStatus == REFLOW_STATUS_ON)
  {
    if (millis() > nextRead1)
    {
      //двигаем звездочки по экрану и готовим данные для вывода на ПК
      nextRead1 = millis() + GRAPHICS_SAMPLING_TIME;
      //закомментировать две строки ниже, если вывод в "serial port plotter"
      sprintf (buf, "OK%03d%03d%03d%03d%03d\r\n", int(Output1), int(Output2), tc1, tc2, currentProfile);
      Serial.print(buf); //это для графиков на ПК

      //раскомментировать строку ниже, если вывод в "serial port plotter"
      //sprintf(buf, "$%d %d;", tc2, tc1); //графики на ПК
      if (curCount <= 6)
      {
        lcd.setCursor(curCount, 0);
        lcd.print(" *");
        lcd.setCursor(18 - curCount, 0);
        lcd.print("* ");
      }
      if (curCount > 6)
      {
        lcd.setCursor(13 - curCount, 0);
        lcd.print("* ");
        lcd.setCursor(curCount + 5, 0);
        lcd.print(" *");
      }
      curCount++;
      if (curCount > 13) curCount = 0;
    }
    if (millis() > nextRead2)
    {
      //считываем данные с термопар и считаем ПИД
      Input1 = Input1 * 0.8 + 0.2 * (max6675_read_temp (thermoCLK, thermoCStop, thermoSO)); //термопара "ВЕРХНЕГО" нагревателя
      Input2 = Input2 * 0.8 + 0.2 * (max6675_read_temp (thermoCLK, thermoCSbott, thermoSO)); //термопара "НИЖНЕГО"  нагревателя
      tc1 = Input1;
      tc2 = Input2;
      if (TopStart)
      {
        Output1 = Pid1(Input1, Setpoint1, kp1, ki1, kd1);
        if (bh_power_auto == 0) Output2 = Pid2(Input2, Setpoint2, kp2, ki2, kd2);
        if (bh_power_auto == 1) Output2 = bh_power;
      }
      else
      {
        Output1 = 0;
        Output2 = Pid2(Input2, Setpoint2, kp2, ki2, kd2);
      }
      nextRead2 = millis() + SENSOR_SAMPLING_TIME;

      lcd.setCursor(16, 2);
      lcd.print("    ");
      lcd.setCursor(16, 2);
      if (isnan(Input1)) {
        lcd.print("Er");
      } else {
        lcd.print(tc1);
      }
      lcd.setCursor(16, 3);
      lcd.print("    ");
      lcd.setCursor(16, 3);
      if (isnan(Input2)) {
        lcd.print("Er");
      } else {
        lcd.print(tc2);
      }
    }
  }
  else
  {
    digitalWrite(RelayPin1, LOW);
    digitalWrite(RelayPin2, LOW);
  }
}


void Dimming()
{
  if (reflowStatus == REFLOW_STATUS_ON)
  {
    OutPWR_TOP();
    OutPWR_BOTTOM();
  }
  //раскомментировать пять строк ниже, если вывод в "serial port plotter"
  //if (Secs >= 100)
  //{
  //  Serial.println(buf);
  //  Secs = 0;
  // } else Secs++;
}

void OutPWR_TOP()
{
  reg1 = Output1 + er1; //pwr- задание выходной мощности в %, er- ошибка округления

  if (reg1 < 50)
  {
    out1 = LOW;

    er1 = reg1 ; //reg- переменная для расчетов
  }
  else
  {
    out1 = HIGH;

    er1 = reg1 - 100;
  }
  digitalWrite(RelayPin1, out1); //пин через который осуществляется дискретное управление

}

void OutPWR_BOTTOM()
{
  reg2 = Output2 + er2; //pwr- задание выходной мощности в %, er- ошибка округления
  if (reg2 < 50)
  {
    out2 = LOW;

    er2 = reg2 ; //reg- переменная для расчетов
  }
  else
  {
    out2 = HIGH;

    er2 = reg2 - 100;
  }

  digitalWrite(RelayPin2, out2); //пин через который осуществляется дискретное управление
}

byte Pid1(double temp, byte ust, byte kP, byte kI, byte kd)
{
  byte out = 0;
  static float ed = 0;
  e1 = (ust - temp); //ошибка регулирования
  p1 =  (kP * e1); //П составляющая
  integra = (integra < i_min) ? i_min : (integra > i_max) ? i_max : integra + (kI * e1) / 1000; //И составляющая
  d1 = kd * (temp - ed); //Д составляющая
  ed = temp;
  out = (p1 + integra + d1 < 0) ? 0 : (p1 + integra + d1 > pwr_TOP) ? pwr_TOP : p1 + integra + d1;
  return out;
}

byte Pid2(double temp, byte ust, byte kP, byte kI, byte kd)
{
  byte out = 0;
  static float ed = 0;
  e2 = (ust - temp); //ошибка регулирования
  p2 =  (kP * e2); //П составляющая
  integra2 = (integra2 < i_min) ? i_min : (integra2 > i_max) ? i_max : integra2 + (kI * e2) / 1000; //И составляющая
  d2 = kd * (temp - ed) * 10; //Д составляющая
  ed = temp;
  out = (p2 + integra2 + d2 < 0) ? 0 : (p2 + integra2 + d2 > pwr_BOTTOM) ? pwr_BOTTOM : p2 + integra2 + d2;
  return out;
}

//---MAX6675 functions---//
double max6675_read_temp (int ck, int cs, int so)
{ char i;
  int tmp = 0;
  digitalWrite(cs, LOW);//cs = 0;                            // Stop a conversion in progress
  asm volatile (" nop"  "\n\t");
  for (i = 15; i >= 0; i--)
  { digitalWrite(ck, HIGH);
    asm volatile (" nop"  "\n\t");
    if ( digitalRead(so))
      tmp |= (1 << i);
    digitalWrite(ck, LOW);
    asm volatile (" nop"  "\n\t");
  }
  digitalWrite(cs, HIGH);
  if (tmp & 0x4) {
    return NAN;
  } else
    return ((tmp >> 3)) * 0.25;
}
//---MAX6675 functions---//

//другие мелодии приветствия
/*
  //Гимн СССР https://clck.ru/FX6CG
    tone(buzzerPin, 174, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 233, 499.9995);
    delay(555.555);
    tone(buzzerPin, 174, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 195, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 220, 499.9995);
    delay(555.555);
    tone(buzzerPin, 146, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 146, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 195, 499.9995);
    delay(555.555);
    tone(buzzerPin, 174, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 155, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 174, 499.9995);
    delay(555.555);
    tone(buzzerPin, 116, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 116, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 130, 499.9995);
    delay(555.555);
    tone(buzzerPin, 130, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 146, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 155, 499.9995);
    delay(555.555);
    tone(buzzerPin, 155, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 174, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 195, 499.9995);
    delay(555.555);
    tone(buzzerPin, 220, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 233, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 261, 749.99925);
    delay(833.3325);
    tone(buzzerPin, 174, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 293, 499.9995);
    delay(555.555);
    tone(buzzerPin, 261, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 233, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 261, 499.9995);
    delay(555.555);
    tone(buzzerPin, 174, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 174, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 233, 499.9995);
    delay(555.555);
    tone(buzzerPin, 220, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 195, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 220, 499.9995);
    delay(555.555);
    tone(buzzerPin, 146, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 146, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 195, 499.9995);
    delay(555.555);
    tone(buzzerPin, 174, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 155, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 174, 499.9995);
    delay(555.555);
    tone(buzzerPin, 116, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 116, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 233, 499.9995);
    delay(555.555);
    tone(buzzerPin, 220, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 195, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 174, 999.999);
    delay(1111.11);
    tone(buzzerPin, 293, 999.999);
    delay(1111.11);
    tone(buzzerPin, 261, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 233, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 220, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 233, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 261, 749.99925);
    delay(833.3325);
    tone(buzzerPin, 174, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 174, 999.999);
    delay(1111.11);
    tone(buzzerPin, 233, 999.999);
    delay(1111.11);
    tone(buzzerPin, 220, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 195, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 174, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 195, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 220, 749.99925);
    delay(833.3325);
    tone(buzzerPin, 146, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 146, 999.999);
    delay(1111.11);
    tone(buzzerPin, 233, 499.9995);
    delay(555.555);
    tone(buzzerPin, 195, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 220, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 233, 499.9995);
    delay(555.555);
    tone(buzzerPin, 195, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 220, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 233, 499.9995);
    delay(555.555);
    tone(buzzerPin, 195, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 233, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 311, 999.999);
    delay(1111.11);
    tone(buzzerPin, 311, 999.999);
    delay(1111.11);
    tone(buzzerPin, 293, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 261, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 233, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 261, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 293, 749.99925);
    delay(833.3325);
    tone(buzzerPin, 233, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 233, 999.999);
    delay(1111.11);
    tone(buzzerPin, 261, 999.999);
    delay(1111.11);
    tone(buzzerPin, 233, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 220, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 195, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 220, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 233, 749.99925);
    delay(833.3325);
    tone(buzzerPin, 195, 249.99975);
    delay(277.7775);
    tone(buzzerPin, 195, 999.999);
    delay(1111.11);
    tone(buzzerPin, 233, 499.9995);
    delay(555.555);
    tone(buzzerPin, 220, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 195, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 174, 499.9995);
    delay(555.555);
    tone(buzzerPin, 116, 374.999625);
    delay(416.66625);
    tone(buzzerPin, 116, 124.999875);
    delay(138.88875);
    tone(buzzerPin, 174, 999.999);
    delay(1111.11);
    tone(buzzerPin, 195, 499.9995);
    delay(555.555);
    tone(buzzerPin, 220, 499.9995);
    delay(555.555);
    tone(buzzerPin, 233, 1999.998);
    delay(2222.22);
*/
/*
  //Мелодия приветствия Марио
  tone(buzzerPin,1318,150);
  delay(150);
  tone(buzzerPin,1318,300);
  delay(300);
  tone(buzzerPin,1318,150);
  delay(300);
  tone(buzzerPin,1046,150);
  delay(150);
  tone(buzzerPin,1318,300);
  delay(300);
  tone(buzzerPin,1568,600);
  delay(600);
  tone(buzzerPin,784,600);
  delay(600);
  noTone(buzzerPin);
*/
